<?php

class Api_advertiser extends CI_Model {

    private $api_key, $hash, $dop_info, $goal_id, $t_info, $g_info;

    function __construct() {
        parent::__construct();
        $this->api_key = htmlspecialchars($this->input->post("api_key"));
        $this->hash = htmlspecialchars($this->input->post("hash"));
        $this->flow_hash = htmlspecialchars($this->input->post("flow_hash"));
        $this->dop_info = strip_tags($this->input->post("dop_info"));
        $this->goal_id = (int) ($this->input->post("goal_id"));
    }

    public function getApiKey() {
        return $this->api_key;
    }

    public function getHash() {
        return $this->hash;
    }

    public function getFlowHash() {
        return $this->flow_hash;
    }

    public function getDopInfo() {
        return $this->dop_info;
    }

    public function getGoalId() {
        return $this->goal_id;
    }

    //Проверка на приличность запроса
    public function isNormal() {
        if (!$this->input->post())
            return false;
        if (!$this->getApiKey() || strlen($this->getApiKey()) != 32)
            return false;
        // if (!$this->getHash() || strlen($this->getHash()) != 32)
        //      return false;
        return true;
    }

    //Есть ли такой api_key в бд
    public function hasApiKeyInDb() {
        $query = $this->db->get_where("users", array("api_key" => $this->getApiKey()));
        return $query;
    }

    //Получаем всю информацию о переходе
    public function getHashInfo() {
        //Получаем информацию о цели (цену)
        $query = $this->db->get_where("goals", array("id" => $this->getGoalId()));
        if ($query->num_rows() == 0) {
            //Цель отсутствует
            return false;
        }
        $this->g_info = $query->row();
        $profit = $query->row()->price;
        $real_profit = $query->row()->real_price;
        $transitQuery = $this->db->get_where("transits", array("hash" => $this->getHash()));
        if ($transitQuery->num_rows() == 0) {
            //Такой hash отсутствует
            return false;
        }
        $this->t_info = $transitQuery->row();
        return array(
            "transit" => $transitQuery->row()
            , "profit" => $profit
            , "real_profit" => $real_profit
        );
    }

    //Добавляем информацию в requests
    public function insertNewRequest($arr = array()) {
        $next_id = $this->db->last_id("requests");
        
        foreach ($arr AS $key => $value) {
            $this->db->set($key, $value);
        }
        $this->db->set("date", date("Y-m-d"));
        $this->db->set("time", date("H:i:s"));

        $this->db->insert("requests");

        
        /*
         * POSTBACK
         */
        if (isset($arr["flow_id"])) {
            $query = $this->db->get_where("flows", array("id" => $arr["flow_id"]));
            if ($query->num_rows() > 0) {
                $row = $query->row();
                if (trim($row->postback_url) != "") {
                    $url = $row->postback_url;
                    $this->send_postback($url);
                }
            }
        }
    }

  
    /*
     * ПЕРЕДАВАЕМЫЕ ДАННЫЕ
     * flow_id, ip, time, datetime, goal_id, goal_name, profit, status, offer_id, offer_name
     */

    private function send_postback($url) {

        $postInfo = "";

        $offer = $this->db->select("name")->from("offers")->where("id", $this->t_info->offer_id)->get();

        $str1 = array(
            "{flow_id}", "{ip}", "{lead_time}", "{lead_date}", "{goal_id}", "{goal_name}",
            "{profit}", "{offer_id}", "{offer_ name}"
        );
        $str2 = array(
            $this->t_info->flow_id, $this->t_info->ip, time(), date("Y-m-d H:i:s"), $this->g_info->id, $this->g_info->name,
            $this->g_info->price, $this->t_info->offer_id, $offer->row()->name
        );

        $infoStr = explode("?", $url);
        $infoArray = explode("&", $infoStr[1]);

        foreach ($infoArray AS $info) {
            $vars = explode("=", $info);
            $postInfo .= $vars[0] . "=" . str_replace($str1, $str2, $vars[1]) . "&";
        }

        $ch = curl_init($infoStr[0]);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_REFERER, config_item("base_url"));
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postInfo);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($ch);
        curl_close($ch);
        return $result;
    }

    public function addTransit($data = array()) {

        $query = $this->db->select("flows.*, pages.url AS page_url, gaskets.url AS gasket_url, gaskets.id AS gasket_id, offers.countries")
                ->where("flows.url", $this->flow_hash)
                ->from("flows")
                ->join("pages", "pages.id=flows.page_id", "left")
                ->join("gaskets", "gaskets.id=flows.gasket_id", "left")
                ->join("offers", "offers.id=flows.offer_id", "left")
                ->get();
        if ($query->num_rows() == 0)
            return false;
        $data["flow_id"] = $query->row()->id;
        $data["page_id"] = $query->row()->page_id;
        $data["offer_id"] = $query->row()->offer_id;
        if ($query->row()->gasket_id)
            $data["gasket_id"] = $query->row()->gasket_id;
        else
            $data["gasket_id"] = 0;
        $data["user_id"] = $query->row()->user_id;
        $data["hash"] = random_string('unique');

        $this->db->insert("transits", $data);
        /*
         * TRAFFICBACL
         */
        if (trim($query->row()->trafficback_url) != "") {
            $c_array = explode(", ", $query->row()->countries);
            $codes = config_item("countries_code");
            if (!in_array($data["country_code"], $codes) || !in_array($codes[$data["country_code"]], $c_array))
                return array("trafficback_url" => $query->row()->trafficback_url, "transit_hash" => $data["hash"]);
        }
        /*
         * END TRAFFICBACL
         */

        return $data["hash"];
    }

}
